import geopandas as gpd
from shapely.geometry import Polygon, box
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import json

def extract_bounding_boxes_from_geojson(file_path_json):
    # Load GeoJSON file using geopandas
    gdf = gpd.read_file(file_path_json)

    # Assuming the GeoJSON contains a MultiPolygon and we are interested in its coordinates
    eluru_coordinates = []

    # Iterate through each geometry in the GeoDataFrame
    for geometry in gdf.geometry:
        if geometry.geom_type == 'Polygon':
            coords = list(geometry.exterior.coords)
            eluru_coordinates.extend(coords)
        elif geometry.geom_type == 'MultiPolygon':
            # Iterate over individual polygons in a MultiPolygon
            for poly in geometry.geoms:
                coords = list(poly.exterior.coords)
                eluru_coordinates.extend(coords)

    # Ensure the polygon is closed by adding the first point again at the end
    if len(eluru_coordinates) > 0:
        eluru_coordinates.append(eluru_coordinates[0])

    eluru_polygon_coords = eluru_coordinates

    # Define the size of each grid cell in pixels
    cell_size_pixels = 24  # Pixel size

    # Conversion factor based on latitude for simplicity
    conversion_factor = 0.0001  # Adjust as per your region and specific needs

    # Convert pixel size to degrees (latitude and longitude)
    cell_size_latlon = cell_size_pixels * conversion_factor

    # Create a Polygon object for the Eluru district
    eluru_polygon = Polygon(eluru_polygon_coords)

    # Determine bounding box of the entire Eluru polygon
    min_x, min_y, max_x, max_y = eluru_polygon.bounds

    # Calculate the range of latitude and longitude
    range_lat = max_y - min_y
    range_lon = max_x - min_x

    # Calculate the number of divisions needed along latitude and longitude axes
    num_divisions_lat = int(range_lat / cell_size_latlon) + 1
    num_divisions_lon = int(range_lon / cell_size_latlon) + 1

    # Initialize an empty dictionary to store bounding boxes
    bounding_boxes = {}

    # Initialize figure and axis for plotting
    fig, ax = plt.subplots(figsize=(10, 10))

    # Plot the Eluru polygon
    x, y = eluru_polygon.exterior.xy
    ax.plot(x, y, color='blue', alpha=0.7, linewidth=2, solid_capstyle='round', zorder=2)

    # Iterate over each grid cell
    for i in range(num_divisions_lon):
        for j in range(num_divisions_lat):
            # Calculate bounding box for current cell in latitude and longitude
            cell_min_lon = min_x + i * cell_size_latlon
            cell_max_lon = min_x + (i + 1) * cell_size_latlon
            cell_min_lat = min_y + j * cell_size_latlon
            cell_max_lat = min_y + (j + 1) * cell_size_latlon

            # Create a box Polygon for the current cell in lat/lon
            cell_bbox = box(cell_min_lon, cell_min_lat, cell_max_lon, cell_max_lat)

            # Intersect the Eluru polygon with the current cell bounding box
            intersected_polygon = eluru_polygon.intersection(cell_bbox)

            # Check if intersection resulted in a valid polygon
            if not intersected_polygon.is_empty:
                # Store bounding box coordinates in dictionary
                bounding_boxes[f"{i}_{j}"] = intersected_polygon.bounds

                # Plot grid cell as a polygon
                grid_polygon = patches.Polygon([(cell_min_lon, cell_min_lat),
                                                (cell_max_lon, cell_min_lat),
                                                (cell_max_lon, cell_max_lat),
                                                (cell_min_lon, cell_max_lat)],
                                               edgecolor='gray', linewidth=0.5, facecolor='none', zorder=1)
                ax.add_patch(grid_polygon)

    # Show the plot
    plt.show()
    plt.savefig("west_godavari_grid_bboxes.svg")

    file_path = "west_godavari_grid_bboxes.json"
    with open(file_path, 'w') as file:
        json.dump(bounding_boxes, file)
    file.close()

    print(f"{len(bounding_boxes.keys())} bounding boxes were generated to extract images of Eluru District.")
    print(f"These bounding boxes are saved at {file_path}.")
    return bounding_boxes

bounding_boxes = extract_bounding_boxes_from_geojson("west_godavari.json")
